@@ -38,6 +38,7 @@ from Service import Service
 from FailoverDomain import FailoverDomain
 from Device import Device
 from Method import Method
+from Vm import Vm
 from Fence import Fence
 from Lockserver import Lockserver
 from ResourceHandler import ResourceHandler
@@ -56,6 +57,7 @@ FENCE_OBJ_COL=3
 
 NODE_NEW = 0
 NODE_EXISTING = 1
+NFSCLIENT_TAG="nfsclient"
 
 FI_TYPE=_("Fence Device Type: \n %s")
 
@@ -90,6 +92,8 @@ CONFIRM_FD_DELETE=_("Are you certain that you wish to delete Fence Device %s?")
 
 ADD_FENCE=_("Add a New Fence")
 
+UNABLE_LOCATE_VM=_("The Virtual Server to be edited, %s, can not be located")
+
 EDIT_FENCE=_("Edit Properties for Fence: %s")
 
 SELECT_LEVEL_FOR_FENCE=_("Please select a fence level for the new fence first.")
@@ -122,6 +126,12 @@ CONFIRM_RC_REMOVE=_("Are you certain that you wish to remove resource %s ?")
 
 CONFIRM_SVC_REMOVE=_("Are you certain that you wish to remove service %s ?")
 
+CONFIRM_VM_REMOVE=_("Are you certain that you wish to remove virtual service %s ?")
+
+VM_NAME_REQUIRED=_("Please provide a name for this virtual service.")
+
+VM_PATH_REQUIRED=_("Please provide a path to the config file for this virtual service.")
+
 SERVICE_NAME_REQUIRED=_("Please provide a name for this service.")
 
 UNIQUE_SERVICE_NAME=_("Services must have unique names. The name %s is already in use. Please provide a unique name.")
@@ -239,6 +249,7 @@ class ConfigTabController:
     self.cluster_props_dlg = self.glade_xml.get_widget('cluster_props')
     self.cluster_props_dlg.connect("delete_event", self.cluster_props_dlg_delete)
     self.clustername = self.glade_xml.get_widget('clustername')
+    self.actualclustername = self.glade_xml.get_widget('actualclustername')
     self.config_version = self.glade_xml.get_widget('config_version')
     self.post_join = self.glade_xml.get_widget('post_join')
     self.post_fail = self.glade_xml.get_widget('post_fail')
@@ -279,6 +290,18 @@ class ConfigTabController:
     self.glade_xml.get_widget('service_delete_b').connect('clicked',self.on_svc_del)
 
 
+    #VMs and Virtual Services button panels
+    self.vm_props_dlg = self.glade_xml.get_widget('vm_props_dlg')
+    self.vm_props_dlg.connect('delete_event',self.on_vm_props_delete)
+    self.glade_xml.get_widget('okbutton21').connect("clicked",self.on_vm_props_ok)
+    self.glade_xml.get_widget('cancelbutton21').connect("clicked",self.on_vm_props_cancel)
+    self.vm_name = self.glade_xml.get_widget('vm_name')
+    self.vm_name_label = self.glade_xml.get_widget('vm_name_l')
+    self.vm_path = self.glade_xml.get_widget('vm_path')
+    self.glade_xml.get_widget('vm_add_b').connect('clicked',self.on_vm_add)
+    self.glade_xml.get_widget('vm_edit_b').connect('clicked',self.on_vm_edit)
+    self.glade_xml.get_widget('vm_delete_b').connect('clicked',self.on_vm_del)
+
 
     #Fence Devices
     self.glade_xml.get_widget('fencedevices_add_b').connect('clicked',self.on_fd)
@@ -369,7 +392,9 @@ class ConfigTabController:
 
     #set fields for edit
     name = cptr.getNameAlias()
+    actualname = cptr.getName()
     self.clustername.set_text(name)
+    self.actualclustername.set_text(actualname)
     self.config_version.set_text(cptr.getConfigVersion())
 
     #set fence daemon fields
@@ -628,6 +653,8 @@ class ConfigTabController:
         kees = attrlist.keys()
         for k in kees:
           f_obj.addAttribute(k,attrlist[k])
+        if agent_type == "fence_scsi":
+          f_obj.addAttribute("node",nd.getName())
         level_obj.addChild(f_obj)
         self.prep_fence_panel(nd)
         self.fi_panel.hide()
@@ -652,6 +679,15 @@ class ConfigTabController:
           except KeyError, e:
             pass    #no attr, no worries
  
+        #special case for ipmi lanplus...if lanplus cbox is unchecked, attr must be as well
+        if agent_type == "fence_ipmilan":
+          try:
+            check_var = attrlist["lanplus"]
+            if check_var == "":
+              f_obj.removeAttribute("lanplus") #This rm's any old lanplus
+          except KeyError, e:
+            pass    #no attr, no worries
+ 
 
     self.model_builder.setModified()
     #self.fi_panel.hide()
@@ -884,6 +920,8 @@ class ConfigTabController:
       cn = ClusterNode()
       cn.addAttribute(NAME_ATTR,nameattr)
       cn.addAttribute(VOTES_ATTR,votesattr)
+      nodeid = self.model_builder.getUniqueNodeID()
+      cn.addAttribute(NODEID_ATTR,nodeid)
       self.model_builder.addNode(cn)
 
       if self.model_builder.getLockType() == GULM_TYPE:
@@ -1417,6 +1455,14 @@ class ConfigTabController:
         self.rc_panel.hide()
         for k in returnlist.keys():
           r_obj.addAttribute(k, returnlist[k])
+        if r_obj.getTagName() == NFSCLIENT_TAG:
+          #If path attr has been removed, we need to remove from obj as well
+          try:
+            pth = returnlist["path"]
+            if pth == "":
+              r_obj.removeAttribute("path")
+          except KeyError, e:
+            r_obj.removeAttribute("path")
         self.model_builder.updateReferences()
         self.model_builder.setModified()
         args = list()
@@ -1443,12 +1489,93 @@ class ConfigTabController:
     self.rc_panel.hide()
     self.rc_handler.clear_rc_forms()
 
+  def on_vm_add(self, button):
+    #launch dialog for virtual service creation
+    self.vm_name.set_sensitive(True)
+    self.vm_name.set_text("")
+    self.vm_path.set_text("")
+    self.vm_props_dlg.run()
+
+  def on_vm_edit(self, button):
+    #launch dialog for virtual service editing
+    # find out which virtual service to edit
+    selection = self.treeview.get_selection()
+    model,iter = selection.get_selected()
+    obj = model.get_value(iter, OBJ_COL)
+    self.vm_name.set_text(obj.getName())
+    self.vm_name.set_sensitive(False)
+    path = obj.getAttribute("path")
+    if path != None:
+      self.vm_path.set_text(path)
+    else:
+      self.vm_path.set_text("")
+    self.vm_props_dlg.run()
+
+  def on_vm_del(self, button):
+    selection = self.treeview.get_selection()
+    model,iter = selection.get_selected()
+    obj = model.get_value(iter, OBJ_COL)
+    retval = self.warningMessage(CONFIRM_VM_REMOVE % obj.getName())
+    if (retval == gtk.RESPONSE_NO):
+      return
+    rm_ptr = self.model_builder.getResourceManagerPtr()
+    rm_ptr.removeChild(obj)
+    self.model_builder.setModified()
+    args = list()
+    args.append(VMS_TYPE)
+    apply(self.reset_tree_model, args)
+    
+
   def on_svc_add(self, button):
     #launch dialog for service name field
     self.svc_name.set_text("")
     #self.svc_add_dlg.show()
     self.svc_add_dlg.run()
 
+  def on_vm_props_ok(self, button):
+    vm_name = self.vm_name.get_text().strip()
+    if vm_name == "":
+      self.errorMessage(VM_NAME_REQUIRED)
+      return
+    vm_path = self.vm_path.get_text().strip()
+    if vm_path == "":
+      self.errorMessage(VM_PATH_REQUIRED)
+      return
+
+
+    try:
+      v = self.model_builder.retrieveVMsByName(vm_name)
+    except:
+      v = None  # Does not exist - good
+
+    if self.vm_name.get_property('sensitive') != 0:  #A new VM
+      if v == None: #This is a new VM
+        v = Vm()
+        v.addAttribute("name",vm_name)
+        v.addAttribute("path",vm_path)
+        rm_ptr = self.model_builder.getResourceManagerPtr()
+        rm_ptr.addChild(v)
+      else:  #Name collision
+        self.vm_name.select_region(0, (-1))
+        self.errorMessage(UNIQUE_VM_NAME % vm_name)
+        return
+
+    else:
+      if v != None:
+        v.addAttribute("path",vm_path)
+      else:
+        self.errorMessage(UNABLE_LOCATE_VM % vm_name)
+        return
+
+    self.vm_props_dlg.hide()
+    self.model_builder.setModified()
+    args = list()
+    args.append(VMS_TYPE) 
+    apply(self.reset_tree_model, args)
+
+  def on_vm_props_cancel(self, button):
+    self.vm_props_dlg.hide()
+
   def on_svc_add_ok(self, button):
     svcs = self.model_builder.getServices()
     svc_name = self.svc_name.get_text().strip()
@@ -1516,6 +1643,7 @@ class ConfigTabController:
 
 
 
+
   def on_f_props_expose_event(self, widget, event):
      self.fence_prop_renderer.do_render()
 
@@ -1599,3 +1727,7 @@ class ConfigTabController:
   def on_svc_add_delete(self, *args):
     self.svc_add_dlg.hide()
     return True
+
+  def on_vm_props_delete(self, *args):
+    self.vm_props_dlg.hide()
+    return True
