@@ -45,6 +45,8 @@ class Device(TagObject):
 
     for attr in attrlist:
       NAME = self.pretty_name_attrs[attr]
+      if self.getAttribute(attr) == None:
+        continue
       VALUE = cgi.escape(self.getAttribute(attr))
       stringbuf = stringbuf + "<span><b>" + NAME + ": " + "</b>" + VALUE + "\n</span>"
 
